import 'package:flutter/material.dart';
import 'package:window_manager/window_manager.dart';

import '../models/control.dart';
import 'create_control.dart';
import 'error.dart';

class WindowDragAreaControl extends StatelessWidget {
  final Control? parent;
  final Control control;
  final List<Control> children;
  final bool parentDisabled;

  const WindowDragAreaControl(
      {Key? key,
      this.parent,
      required this.control,
      required this.children,
      required this.parentDisabled})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    debugPrint("WindowDragArea build: ${control.id}");

    var contentCtrls =
        children.where((c) => c.name == "content" && c.isVisible);
    bool maximizable = control.attrBool("maximizable", true)!;
    bool disabled = control.isDisabled || parentDisabled;

    if (contentCtrls.isEmpty) {
      return const ErrorControl("WindowDragArea should have content.");
    }

    return constrainedControl(
        context,
        WindowDragArea(
            maximizable: maximizable,
            child: createControl(control, contentCtrls.first.id, disabled)),
        parent,
        control);
  }
}

class WindowDragArea extends StatelessWidget {
  final Widget child;
  final bool maximizable;

  const WindowDragArea(
      {Key? key, required this.child, required this.maximizable})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      behavior: HitTestBehavior.translucent,
      onPanStart: (details) {
        windowManager.startDragging();
      },
      onDoubleTap: maximizable
          ? () async {
              bool isMaximized = await windowManager.isMaximized();
              if (!isMaximized) {
                windowManager.maximize();
              } else {
                windowManager.unmaximize();
              }
            }
          : null,
      child: child,
    );
  }
}
